#include "main.h"

u8 key_state;				// debounced and inverted key state:
					// bit = 1: key pressed
u8 key_press;				// key press detect

u8 flash_timer;				// Timers
u8 sleep_timer;

#define TIMEOUT		50		// 50 * 10ms = 500ms
#define	FLASH_ON	25
#define	FLASH_OFF	50


void get_key_init( void )
{
  TCCR0A = 0;				// Mode 0
  TCCR0B = 1<<CS02^1<<CS00;		// divide by 1024
  TIMSK0 = 1<<TOIE0;			// enable timer interrupt
}


u8 get_key_press( u8 key_mask )
{
  cli();				// read and clear atomic !
  key_mask &= key_press;                // read key(s)
  key_press ^= key_mask;                // clear key(s)
  sei();
  return key_mask;
}


int main( void )
{
  u8 state = 0;

  PORTB = 0xFF;				// all IOs high (LEDs off, Pullups on)
  LED0_DDR = 1;
  LED1_DDR = 1;
  PCMSK = 1<<KEY0;			// PCINT on Key0
  get_key_init();
  sei();

  for(;;){

    if( get_key_press( 1<<KEY0 ) )
      state++;				// next state on every key press

    switch( state ){
      case 2:				// LED flashing
	sleep_timer = 0;				// no sleep on flashing
	if( flash_timer != FLASH_ON ){			// on time
	  if( flash_timer < FLASH_ON + FLASH_OFF )	// off time
	    break;

      case 1:
	  flash_timer = 0;
	  LED0 = 0;			// LED on
	  break;

      case 0:
      default:
	  state = 0;
	}
	LED0 = 1;			// LED off
	break;
    }

    LED1 = 1;				// for debug only
					// LED1 = off: power down
    sleep_cpu();
    LED1 = 0;
  }
}


ISR( PCINT0_vect )
{
  GIMSK = 0;				// disable PCINT
  MCUCR = 0;				// disable power down
  sleep_timer = 0;			// reset sleep timer
}


ISR( TIM0_OVF_vect )			// every 10ms
{
  static u8 ct0, ct1;
  u8 i;

  TCNT0 = (u8)(s16)-(XTAL / 1024 * 10e-3 + 0.5);	// preload for 10ms
/******************************	Timers ***********************************/
  flash_timer++;
  if( sleep_timer < TIMEOUT ){
    sleep_timer++;
  }else{
    MCUCR = 1<<SE^1<<SM1;		// enable power down
  }
  GIMSK = 1<<PCIE;		      	// enable Pin Change
/******************************	Key debounce *****************************/
  i = key_state ^ ~KEY_PIN;		// key changed ?
  ct0 = ~( ct0 & i );			// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);		// reset or count ct1
  i &= ct0 & ct1;			// count until roll over ?
  key_state ^= i;			// then toggle debounced state
  key_press |= key_state & i;		// 0->1: key press detect
}
